# Bayes model for Site Index - comparison
#-------------------------------------------

rm(list = ls())

dir_name = "Code R/"

if (dir.exists("figsOutBayes")){
  dir.create(paste0(dir_name, "figsOutBayes"))
}

if (dir.exists("dataOutBayes")){
  dir.create(paste0(dir_name, "dataOutBayes"))
}

# Libraries and packages
#----------------------------------------------------------------------------------------
source(paste0(dir_name, "_helper_functions/pckgLoad.R"))

pckgs = readLines(paste0(dir_name, "_R_list_packages.txt"))
pckgs = gsub("^#.*", "", pckgs) # remove the coments in the text file 
pckgs = pckgs[pckgs != ""] # remove the "" entries

pckgLoad(pckgs)

# Load functions
#----------------------------------------------------------------------------------------

fun.list = list.files(path = paste0(dir_name, "functions/"))
fun.list = paste0(dir_name, "functions/", fun.list)

for (nm in fun.list){
  source(nm)
}

#-----------------------------------------------------------------------------------------
#input data
#...........

B_vals = c(21.9, 26.9, 31.8, 36.9)

data_plot_all = data.frame()
data_table_all = data.frame()
data_table_age_all = data.frame()

molidz = read_xlsx(paste0(dir_name, "dataIn/ZeletinMolid.xlsx"), sheet="MolidZeletin80")

molidz = molidz %>% group_by(age, B) %>%
  mutate(r = 1:n())

for (j in seq_along(B_vals)){
    
  keep_B = B_vals[j]
  
  n_samples = 20
  
  molidz_sampled = molidz %>% group_by(age, B) %>% 
    filter(r %in% sample(1:max(r), n_samples))
  
  molidz_sampled$index = 1:length(molidz_sampled$B)
  
  molidz_sampled = molidz_sampled %>% 
    filter(B %in% keep_B) %>%
    mutate(tn = tanh(h/B),
           aget = 1/sqrt(age))
  
  # Extract model information including model matrix X, response y 
  #................................................................
  model.molid = lm(tn ~ aget, data = molidz_sampled)
  shapiro.test(residuals(model.molid))
  
  Yh = model.molid$fitted.values
  s = summary(model.molid)$sigma
  ns = length(model.molid$residuals)
  
  myh = mean(Yh)

  Nterm = c(5, 10, 20, 30)
  my = rep(0, length(Nterm))
  
  # create data
  Y1 = molidz_sampled$h
  data = Y1
  
  Y2 = NULL
  
  for (i in seq_along(Nterm)){
    
    Yi = sapply(Yh, function(x){tmY_HyperTangent_c(mu = x, sigma = s, nsample = ns, nterm = Nterm[i])})
    data = cbind(data, Yi)
    
    Y2 = c(Y2, Yi)
    my[i] = mean(Yi)
  }
  
  
  data = as.data.frame(data)
  names(data) = c("Initial", paste0("n = ", Nterm))
  
  data$age = molidz_sampled$age
  data$aget = molidz_sampled$aget
  
  data$B = molidz_sampled$B
  
  
  # computations with stan
  #............................

  yt = molidz_sampled$tn
  N = length(molidz_sampled$tn) # new N
  x = molidz_sampled$aget # new x after rejection 
  
  fit_tanh = stan(file = paste0(dir_name, "Stan models/Transform_tanh.stan"), 
                  data = c("N", "x", "yt"), 
                  chains = 4, 
                  cores = 2, 
                  iter = 5000, 
                  warmup = 2000) # for speed
  
  fit_tanh_summary = summary(fit_tanh)
  
  y_bayes_mean = fit_tanh_summary$summary[paste0("ypredb[", 1:N, "]"), "mean"]
  y_bayes_lb = fit_tanh_summary$summary[paste0("ypredb[", 1:N, "]"), "2.5%"]
  y_bayes_ub = fit_tanh_summary$summary[paste0("ypredb[", 1:N, "]"), "97.5%"]
  
  data$Bayes = y_bayes_mean
  data$Bayes_lb = y_bayes_lb
  data$Bayes_ub = y_bayes_ub
  
  # Model comparison - Giurgiu and Draghiciu (2004) Model
  #........................................................
  giurgiu = nlsLM(h ~ b0*exp(b1/age^b2), data = molidz_sampled, start=list(b0=30, b1=-40, b2=1))
  
  giurgiu.cc = cor(predict(giurgiu), molidz_sampled$h)
  giurgiu.r2 = 1 - sum(resid(giurgiu)^2)/(length(molidz_sampled$h)*var(molidz_sampled$h))
  
  data$Giurgiu = predict(giurgiu)
  
  data_plot2 = data %>% 
    select(-c(Bayes_lb, Bayes_ub)) %>%
    pivot_longer(-c(Initial, age, aget, B), names_to = "Model", values_to = "y") %>%
    mutate(y = ifelse(Model == "Giurgiu", y, y*B))
  
  data_plot2$Model = fct_relevel(as_factor(data_plot2$Model), c("Giurgiu", "Bayes", paste0("n = ", Nterm)))
  levels(data_plot2$Model) = c("Giurgiu", "Bayes",  paste0("model - ", Nterm))
    
  pl2 = ggplot(data_plot2, aes(x = age))+
    geom_point(aes(y = Initial), colour = "darkgrey", size = 1.5) +
    geom_line(aes(x = age, y = y, colour = Model), alpha = 0.8, size = 1.5)+
    theme_minimal()+
    labs(x = "age", y = "Back-transformed", colour = "Model - no. of terms") + 
    scale_color_ptol()+
    theme(legend.position = "bottom")
  
  pl2
  
  pl3 = ggplot()+
    geom_point(data = data_plot2, aes(x = age, y = Initial), colour = "darkgrey", size = 1.5) +
    geom_line(data = data_plot2%>% filter(Model != "Bayes"), 
              aes(x = age, y = y, colour = Model), alpha = 0.8, size = 1.5)+
    geom_line(data = data_plot2%>% filter(Model == "Bayes"), 
              aes(x = age, y = y, colour = Model), alpha = 0.8, size = 1.7, linetype = 2)+
    theme_minimal()+
    labs(x = "age", y = "Back-transformed", colour = "Model - no. of terms") + 
    scale_color_ptol()+
    theme(legend.position = "bottom") 
  
  
  ggsave(paste0(dir_name, "figsOutBayes/Fig_SI_(B=", keep_B, ").pdf"), plot = pl3, 
         width=11, height=8.5)
  
  
  pl4 = ggplot()+
    geom_point(data = data_plot2, aes(x = age, y = Initial), colour = "darkgrey", size = 1.7) +
    geom_smooth(data = data_plot2 %>% filter(Model != c("Bayes", "Giurgiu")), 
                aes(x = age, y = y, colour = Model, linetype = Model), method = "loess",
                alpha = 0.8, size = 1.5, se = FALSE)+
    geom_smooth(data = data_plot2 %>% filter(Model %in% c("Bayes", "Giurgiu")), 
                aes(x = age, y = y, colour = Model, linetype = Model, size = Model), method = "loess",
                alpha = 0.8,  se = FALSE)+ 
    scale_linetype_manual(values=c("dotted", "dotdash", rep("solid", 4)))+
    scale_size_manual(values = c(1.8, 1.8), guide = "none") +
    theme_minimal()+
    labs(x = "age", y = "Back-transformed", colour = "Model - no. of terms", linetype = "Model - no. of terms") + 
    scale_color_ptol()+
    theme(legend.position = "bottom")+
    guides(col = guide_legend(nrow=2, byrow=TRUE)) 
  
  ggsave(paste0(dir_name, "figsOutBayes/Fig_SI_Bayes_(B=", keep_B, ").pdf"), plot = pl4, 
         width=11, height=8.5)
  
  # Comparison data
  #..................

  # correlations
  model.cc = data_plot2 %>% 
    group_by(Model) %>%
    summarise(Correlation = cor(Initial, y))
  
  model.cc.age = data_plot2 %>% 
    group_by(age, Model) %>%
    summarise(correlation = cor(Initial, y))
  
  dataini = data %>% group_by(age) %>%
    summarise(Initial_Mean_Age = mean(Initial)) %>%
    left_join(data, by = "age")
  
  
  #statistics: bias, MAB and RMSE
  dataini %>% group_by(age) %>%
    summarise(data_bias = mean(Initial_Mean_Age - Initial),
              data_MAB = mean(abs(Initial_Mean_Age - Initial)),
              data_RMSE = mean((Initial_Mean_Age - Initial)^2)^0.5)
  
  data_stat = dataini %>% 
    summarise(data_bias = mean(Initial_Mean_Age - Initial),
              data_MAB = mean(abs(Initial_Mean_Age - Initial)),
              data_RMSE = mean((Initial_Mean_Age - Initial)^2)^0.5)
  
  # by age
  data_table_stats_age = data_plot2 %>% 
    group_by(age, Model) %>%
    summarise(bias = mean(Initial - y),
              rel_bias = mean((Initial - y)/Initial),
              MAB = mean(abs(Initial - y)),
              rel_MAB = mean(abs(Initial - y)/Initial),
              RMSE = mean((Initial - y)^2)^0.5,
              rel_RMSE = mean((abs(Initial - y)/Initial)^2)^0.5)
  
  write.csv(data_table_stats_age, file = paste0(dir_name, "dataOutBayes/Table_SI_age(B=", keep_B, ").csv"))
  
  data_table_stats_age$B = keep_B
  data_table_age_all = bind_rows(data_table_age_all, data_table_stats_age)
  
  # all
  data_table_stats = data_plot2 %>% 
    group_by(Model) %>%
    summarise(Bias = mean(Initial - y),
              Rel_Bias = mean((Initial - y)/Initial),
              MAB = mean(abs(Initial - y)),
              Rel_MAB = mean(abs(Initial - y)/Initial),
              RMSE = mean((Initial - y)^2)^0.5,
              Rel_RMSE = mean((abs(Initial - y)/Initial)^2)^0.5) %>%
    left_join(model.cc, by = "Model")
  
  
  write.csv(data_table_stats, file = paste0(dir_name, "dataOutBayes/Table_SI_(B=", keep_B, ").csv"))
}
