# Script for Taper Model of loblolly pine
#-------------------------------------------

rm(list = ls())

dir_name = "Code R/"

if (dir.exists("figsOutBayes")){
  dir.create(paste0(dir_name, "figsOutBayes"))
}

if (dir.exists("dataOutBayes")){
  dir.create(paste0(dir_name, "dataOutBayes"))
}

# Libraries and packages
#----------------------------------------------------------------------------------------
source(paste0(dir_name, "_helper_functions/pckgLoad.R"))

pckgs = readLines(paste0(dir_name, "_R_list_packages.txt"))
pckgs = gsub("^#.*", "", pckgs) # remove the coments in the text file 
pckgs = pckgs[pckgs != ""] # remove the "" entries

pckgLoad(pckgs)

# Load functions
#----------------------------------------------------------------------------------------

fun.list = list.files(path = paste0(dir_name, "functions/"))
fun.list = paste0(dir_name, "functions/", fun.list)

for (nm in fun.list){
  source(nm)
}


#-----------------------------------------------------------------------------------------
library(rstan)
library(shinystan)

#input data
#...........

taper = read_xlsx(paste0(dir_name, "dataIn/LoblollyLA.xlsx"))

taper = taper %>% group_by(TreeID) %>%
  mutate(dbh = D_cm[Height == 1.3],
         D_cm1 = D_cm/(2*dbh),
         Ht = Height/HeightTotal, 
         Ht_x1 = log(HeightTotal)*sqrt(Ht), 
         # Ht_x2 = HeightTotal/sqrt(dbh)) # <- this is the actual model
         Ht_x2 = HeightTotal/sqrt(D_cm)) 

model.taper = lm(cos(D_cm1) ~ Ht_x1 + Ht_x2, data = taper)
# summary(model.taper)
# cor(predict(model.taper),taper$D_cm1)
# shapiro.test(residuals(model.taper))

# Extract model information including model matrix X, response y 
#................................................................

Yh = model.taper$fitted.values
s = summary(model.taper)$sigma
ns = length(model.taper$residuals)

myh = mean(Yh)

Nterm = c(5, 10, 20, 30)
my = rep(0, length(Nterm))

# create data

Y1 = taper$D_cm1 # taper$D_cm/(2*taper$dbh)
data = Y1

Y2 = NULL

for (i in seq_along(Nterm)){
  
  Yi = sapply(Yh, function(x){tmY_cosine_c(mu = x, sigma = s, nsample = ns, nterm = Nterm[i])})
  data = cbind(data, Yi)
  
  Y2 = c(Y2, Yi)
  my[i] = mean(Yi)
}

data = as.data.frame(data)

names(data) = c("Initial", paste0("n = ", Nterm))

data$Ht = taper$Ht
data$Height = taper$Height

# computations with stan
#............................

yt = cos(taper$D_cm1)
N = length(taper$D_cm1) # new N
x1 = taper$Ht_x1 # new x after rejection 
x2 = taper$Ht_x2 # new x after rejection 

fit_cos = stan(file = paste0(dir_name, "Stan models/Transform_cos.stan"), 
                data = c("N", "x1", "x2", "yt"), 
                chains = 4, 
                cores = 2, 
                iter = 5000, 
                warmup = 2000) # for speed

fit_cos_summary = summary(fit_cos)


y_bayes_mean = fit_cos_summary$summary[paste0("ypredb[", 1:N, "]"), "mean"]
y_bayes_lb = fit_cos_summary$summary[paste0("ypredb[", 1:N, "]"), "2.5%"]
y_bayes_ub = fit_cos_summary$summary[paste0("ypredb[", 1:N, "]"), "97.5%"]

data$Bayes = y_bayes_mean

# Model comparison - Kozak 02 Model
#........................................................

taper_tr = transform(taper, 
                     x = (1-Ht^(1/3))/(1-(1.3/HeightTotal)^(1/3)), 
                     q = (1-Ht^(1/3)))

kozak02 <- nls(D_cm1 ~ a0*D_cm^a1*HeightTotal^a2*x^(b1*Ht^4+b2*exp(-(D_cm/HeightTotal))+b3*x^0.1+b4/D_cm+b5*HeightTotal^q+b6*x),
               data = taper_tr,
               start = list(a0 = 1, a1 = 1, a2 = 0.1, b1 = 1, b2 = 1, b3 = 1, b4 = 1, b5 = 1, b6 = 1))

# summary(kozak02)

kozak.r2 = cor(predict(kozak02), taper$D_cm1)

data$Kozak02 = predict(kozak02)

data_plot = data %>% 
  pivot_longer(-c(Initial, Ht, Height), names_to = "Model", values_to = "y") 

data_plot$Model = fct_relevel(as_factor(data_plot$Model), c("Kozak02", "Bayes", paste0("n = ", Nterm)))
levels(data_plot$Model) = c("Kozak02", "Bayes",  paste0("model - ", Nterm))

pl1 = ggplot(data_plot, aes(x = Ht))+
  geom_point(aes(y = Initial), colour = "darkgrey", size = 1.5) +
  geom_smooth(aes(x = Ht, y = y, colour = Model), alpha = 0.8, size = 1.5, se = FALSE)+
  theme_minimal()+
  labs(x = "Relative Height", y = "Relative DBH", colour = "Model - no. of terms") + 
  scale_color_ptol()+
  theme(legend.position = "bottom")+
  guides(col = guide_legend(nrow=2,byrow=TRUE))

pl1

pl2 = ggplot()+
  geom_point(data = data_plot, aes(x = Ht, y = Initial), colour = "darkgrey", size = 1.7) +
  geom_smooth(data = data_plot %>% filter(Model != c("Bayes", "Kozak02")), 
            aes(x = Ht, y = y, colour = Model, linetype = Model), method = "loess", 
            alpha = 0.8, size = 1.5, se = FALSE)+
  geom_smooth(data = data_plot %>% filter(Model %in% c("Bayes", "Kozak02")), 
            aes(x = Ht, y = y, colour = Model, linetype = Model, size = Model), method = "loess",
            alpha = 0.8,  se = FALSE)+ 
  scale_linetype_manual(values=c("dotted", "dotdash", rep("solid", 4)))+
  scale_size_manual(values = c(1.8, 1.8), guide = "none") +
  theme_minimal()+
  labs(x = "Relative Height", y = "Relative DBH", colour = "Model - no. of terms", linetype = "Model - no. of terms") + 
  scale_color_ptol()+
  theme(legend.position = "bottom")+
  guides(col = guide_legend(nrow=2, byrow=TRUE)) 

ggsave(paste0(dir_name, "figsOutBayes/Fig_Taper_Bayes.pdf"), plot = pl2, 
       width=11, height=8.5)

# Comparison data
#..................

# correlations
model.cc.taper = data_plot %>% 
  group_by(Model) %>%
  summarise(Correlation = cor(Initial, y))

# all
data_table_stats_taper = data_plot %>% 
  group_by(Model) %>%
  summarise(Bias = mean(Initial - y),
            Rel_Bias = mean((Initial - y)/Initial),
            MAB = mean(abs(Initial - y)),
            Rel_MAB = mean(abs(Initial - y)/Initial),
            RMSE = mean((Initial - y)^2)^0.5,
            Rel_RMSE = mean((abs(Initial - y)/Initial)^2)^0.5) %>%
  left_join(model.cc.taper, by = "Model")

write.csv(data_table_stats_taper, file = paste0(dir_name, "dataOutBayes/Table_Taper.csv"))
